@@ -21,6 +21,7 @@
 #include "config.h"
 
 #include <vector>
+#include <unistd.h>
 
 #include "gettext.h"
 #define _(string) gettext(string)
@@ -205,6 +206,8 @@ void dispatch::step()
                 global_dispatch->force_stop();
             }
         }
+    } else {
+        usleep(1000);
     }
 }
 
@@ -538,6 +541,10 @@ void dispatch::receive_cmd(const command& cmd)
             _player->set_pause(!pausing());
         /* notify when request is fulfilled */
         break;
+    case command::step:
+        if (_player)
+            _player->set_step(true);
+        break;
     case command::seek:
         if (_player)
             _player->seek(s11n::load<float>(p) * 1e6f);
