@@ -67,8 +67,10 @@ void player::reset_playstate()
     _drop_next_frame = false;
     _previous_frame_dropped = false;
     _in_pause = false;
+    _recently_seeked = false;
     _quit_request = false;
     _pause_request = false;
+    _step_request = false;
     _seek_request = 0;
     _set_pos_request = -1.0f;
     _video_frame = video_frame();
@@ -259,12 +261,20 @@ int64_t player::step(bool *more_steps, int64_t *seek_to, bool *prep_frame, bool
             _current_pos = _video_pos;
         }
         global_dispatch->set_position(normalize_pos(_current_pos));
-        _need_frame_now = false;
-        _need_frame_soon = true;
-        _previous_frame_dropped = false;
-        *more_steps = true;
-        *prep_frame = true;
         set_current_subtitle_box();
+        _recently_seeked = true;
+        *prep_frame = true;
+        *more_steps = true;
+        return 0;
+    }
+    else if (_recently_seeked)
+    {
+        _recently_seeked = false;
+        _need_frame_now = true;
+        _need_frame_soon = false;
+        _previous_frame_dropped = false;
+        *display_frame = true;
+        *more_steps = true;
         return 0;
     }
     else if (_pause_request)
@@ -418,7 +428,8 @@ int64_t player::step(bool *more_steps, int64_t *seek_to, bool *prep_frame, bool
             int64_t delay = next_frame_presentation_time - _video_pos;
             if (delay > global_dispatch->get_media_input()->video_frame_duration() * 75 / 100
                     && !dispatch::parameters().benchmark()
-                    && !global_dispatch->get_media_input()->is_device())
+                    && !global_dispatch->get_media_input()->is_device()
+                    && !_step_request)
             {
                 msg::wrn(_("Video: delay %g seconds/%g frames; dropping next frame."),
                          float(delay) / 1e6f, 
@@ -428,6 +439,8 @@ int64_t player::step(bool *more_steps, int64_t *seek_to, bool *prep_frame, bool
             if (!_previous_frame_dropped)
             {
                 *display_frame = true;
+                if (_step_request)
+                    _pause_request = true;
                 if (dispatch::parameters().benchmark())
                 {
                     _frames_shown++;
@@ -510,6 +523,15 @@ void player::quit_request()
 void player::set_pause(bool p)
 {
     _pause_request = p;
+    if (!p)
+        _step_request = false;
+}
+
+void player::set_step(bool s)
+{
+    _step_request = s;
+    if (s)
+        _pause_request = false;
 }
 
 void player::seek(int64_t offset)
