@@ -153,7 +153,6 @@ protected:
     int pos_x() const { return 0; }
     int pos_y() const { return 0; }
     GLEWContext* glewGetContext() const { return const_cast<GLEWContext*>(_channel->getWindow()->glewGetContext()); }
-    void make_context_current() { _channel->getWindow()->makeCurrent(); }
     bool context_is_stereo() const { return false; }
     void recreate_context(bool) { }
     void trigger_resize(int, int) { }
@@ -468,7 +467,8 @@ public:
                 && _eq_frame_data.seek_to == -1
                 && !_eq_frame_data.prep_frame
                 && !_eq_frame_data.drop_frame
-                && !_eq_frame_data.display_frame);
+                && !_eq_frame_data.display_frame
+                && !dispatch::pausing());
         if (!more_steps)
         {
             this->exit();
@@ -577,6 +577,9 @@ public:
             case 'p':
                 controller::send_cmd(command::toggle_pause);
                 break;
+            case '.':
+                controller::send_cmd(command::step);
+                break;
             case 'v':
                 /* TODO: cycling video streams is currently not supported with Equalizer.
                  * We would have to cycle the streams in all node players, and thus communicate
