@@ -65,27 +65,11 @@
 gl_thread::gl_thread(video_output_qt* vo_qt, video_output_qt_widget* vo_qt_widget) :
     _vo_qt(vo_qt), _vo_qt_widget(vo_qt_widget),
     _render(false),
-    _activate_next_frame(false),
     _resize(false),
-    _prepare_next_frame(false),
-    _have_prepared_frame(false),
+    _action_activate(false),
+    _action_prepare(false),
     _failure(false)
 {
-    for (int i = 0; i < 2; i++) {
-        for (int p = 0; p < 3; p++) {
-            _next_data[i][p] = NULL;
-            _next_line_size[i][p] = 0;
-        }
-    }
-}
-
-gl_thread::~gl_thread()
-{
-    for (int i = 0; i < 2; i++) {
-        for (int p = 0; p < 3; p++) {
-            std::free(_next_data[i][p]);
-        }
-    }
 }
 
 #ifdef Q_WS_X11
@@ -103,11 +87,6 @@ void gl_thread::set_render(bool r)
     _ptc = 0;
 }
 
-void gl_thread::activate_next_frame()
-{
-    _activate_next_frame = true;
-}
-
 void gl_thread::resize(int w, int h)
 {
     _w = w;
@@ -115,78 +94,57 @@ void gl_thread::resize(int w, int h)
     _resize = true;
 }
 
+void gl_thread::activate_next_frame()
+{
+    _action_mutex.lock();
+    _action_activate = true;
+    _action_cond.wait(_action_mutex);
+    _action_mutex.unlock();
+}
+
 void gl_thread::prepare_next_frame(const video_frame &frame, const subtitle_box &subtitle)
 {
-    _prepare_next_mutex.lock();
+    _action_mutex.lock();
     _next_subtitle = subtitle;
     _next_frame = frame;
-    // Copy the data because the decoder can overwrite the buffers at any time
-    try {
-        if (_next_frame.is_valid()) {
-            for (int i = 0; i < 2; i++) {
-                for (int p = 0; p < 3; p++) {
-                    size_t height = _next_frame.raw_height;
-                    if (_next_frame.layout == video_frame::yuv420p && p > 0)
-                        height /= 2;
-                    size_t data_size = height * _next_frame.line_size[i][p];
-                    if (_next_line_size[i][p] != _next_frame.line_size[i][p]) {
-                        _next_data[i][p] = std::realloc(_next_data[i][p], data_size);
-                        if (data_size > 0 && !_next_data[i][p]) {
-                            throw exc(ENOMEM);
-                        }
-                        _next_line_size[i][p] = _next_frame.line_size[i][p];
-                    }
-                    if (data_size > 0) {
-                        std::memcpy(_next_data[i][p], _next_frame.data[i][p], data_size);
-                    }
-                    _next_frame.data[i][p] = _next_data[i][p];
-                }
-            }
-        }
-    }
-    catch (std::exception& e) {
-        _e = e;
-        _render = false;
-        _failure = true;
-    }
-    _prepare_next_mutex.unlock();
-    _prepare_next_frame = true;
+    _action_prepare = true;
+    _action_cond.wait(_action_mutex);
+    _action_mutex.unlock();
 }
 
 void gl_thread::run()
 {
     try {
+        assert(_vo_qt_widget->context()->isValid());
+        _vo_qt_widget->makeCurrent();
+        assert(QGLContext::currentContext() == _vo_qt_widget->context());
         while (_render) {
-            assert(_vo_qt_widget->context()->isValid());
-            _vo_qt_widget->makeCurrent();
-            if (QGLContext::currentContext() == _vo_qt_widget->context()) {
-                if (_activate_next_frame && _have_prepared_frame) {
-                    _vo_qt->video_output::activate_next_frame();
-                    _have_prepared_frame = false;
-                    _activate_next_frame = false;
-                }
-                if (_resize) {
-                    _vo_qt->reshape(_w, _h);
-                    _resize = false;
-                }
-#ifdef Q_WS_X11
-                GLuint counter;
-                if (GLXEW_SGI_video_sync && glXGetVideoSyncSGI(&counter) == 0)
-                    _display_frameno = counter;
-                else
-                    _display_frameno++;
-#else
-                _display_frameno++;
-#endif
-                _vo_qt->display_current_frame(_display_frameno);
-                if (_prepare_next_frame) {
-                    _prepare_next_mutex.lock();
-                    _vo_qt->video_output::prepare_next_frame(_next_frame, _next_subtitle);
-                    _prepare_next_mutex.unlock();
-                    _prepare_next_frame = false;
-                    _have_prepared_frame = true;
-                }
+            _action_mutex.lock();
+            if (_action_activate) {
+                _vo_qt->video_output::activate_next_frame();
+                _action_activate = false;
+                _action_cond.wake_one();
             }
+            if (_action_prepare) {
+                _vo_qt->video_output::prepare_next_frame(_next_frame, _next_subtitle);
+                _action_prepare = false;
+                _action_cond.wake_one();
+            }
+            _action_mutex.unlock();
+            if (_resize) {
+                _vo_qt->reshape(_w, _h);
+                _resize = false;
+            }
+#ifdef Q_WS_X11
+            GLuint counter;
+            if (GLXEW_SGI_video_sync && glXGetVideoSyncSGI(&counter) == 0)
+                _display_frameno = counter;
+            else
+                _display_frameno++;
+#else
+            _display_frameno++;
+#endif
+            _vo_qt->display_current_frame(_display_frameno);
             _vo_qt_widget->swapBuffers();
             // When the buffer swap returns, the current frame is just now presented on screen.
             _pt_mutex.lock();
@@ -204,6 +162,7 @@ void gl_thread::run()
         _render = false;
         _failure = true;
     }
+    _vo_qt_widget->doneCurrent();
 }
 
 int64_t gl_thread::time_to_next_frame_presentation()
@@ -279,7 +238,6 @@ void video_output_qt_widget::check_gl_thread()
 
 void video_output_qt_widget::start_rendering()
 {
-    QApplication::syncX();
     _gl_thread.set_render(true);
     _gl_thread.start();
     _timer.start(0);
@@ -290,7 +248,6 @@ void video_output_qt_widget::stop_rendering()
     _gl_thread.set_render(false);
     _gl_thread.wait();
     _timer.stop();
-    QApplication::syncX();
 }
 
 void video_output_qt_widget::resizeEvent(QResizeEvent* event)
@@ -353,6 +310,9 @@ void video_output_qt_widget::keyPressEvent(QKeyEvent *event)
         }
         break;
 #endif
+    case Qt::Key_Period:
+        _vo->send_cmd(command::step);
+        break;
     case Qt::Key_V:
         _vo->send_cmd(command::cycle_video_stream);
         break;
@@ -716,10 +676,8 @@ void video_output_qt::create_widget()
     container_layout->setColumnStretch(0, 1);
     delete _container_widget->layout();
     _container_widget->setLayout(container_layout);
-    _container_widget->updateGeometry();
-    _widget->show();
-    _container_widget->show();
-    _container_widget->raise();
+    if (!_container_is_external)
+        _container_widget->show();
     process_events();
 }
 
@@ -735,11 +693,6 @@ GLEWContext* video_output_qt::glewGetContext() const
     return const_cast<GLEWContext*>(&_glew_context);
 }
 
-void video_output_qt::make_context_current()
-{
-    _widget->makeCurrent();
-}
-
 bool video_output_qt::context_is_stereo() const
 {
     return (_format.stereo());
@@ -758,7 +711,10 @@ void video_output_qt::trigger_resize(int w, int h)
 {
     _container_widget->set_recommended_size(w, h);
     _container_widget->updateGeometry();
-    _container_widget->adjustSize();
+    // Process events to propagate the information that the geometry needs updating.
+    process_events();
+    if (!_container_is_external)
+        _container_widget->adjustSize();
 }
 
 void video_output_qt::mouse_set_pos(float dest)
@@ -880,71 +836,62 @@ void video_output_qt::center()
 
 void video_output_qt::enter_fullscreen()
 {
-    if (!_fullscreen)
-    {
-        int screens = dispatch::parameters().fullscreen_screens();
+    if (!_fullscreen) {
+        // If the container is a window, we save its geometry here so that
+        // we can restore it later.
+        if (!_container_is_external)
+            _geom = _container_widget->geometry();
+        // If the container is not yet a window, but embedded in the main window,
+        // we need to make it a window now.
         if (_container_is_external)
-        {
             _container_widget->setWindowFlags(Qt::Window);
-        }
         // Determine combined geometry of the chosen screens.
-        QRect geom(0, 0, 0, 0);
+        int screens = dispatch::parameters().fullscreen_screens();
         int screen_count = 0;
-        for (int i = 0; i < std::min(QApplication::desktop()->screenCount(), 16); i++)
-        {
-            if (screens & (1 << i))
-            {
-                if (screen_count == 0)
-                {
+        QRect geom;
+        for (int i = 0; i < std::min(QApplication::desktop()->screenCount(), 16); i++) {
+            if (screens & (1 << i)) {
+                if (geom.isNull())
                     geom = QApplication::desktop()->screenGeometry(i);
-                }
                 else
-                {
                     geom = geom.united(QApplication::desktop()->screenGeometry(i));
-                }
                 screen_count++;
             }
         }
-        if (screen_count <= 1)
-        {
-            // The single screen case is supported by Qt.
-            _container_widget->setWindowState(_widget->windowState() | Qt::WindowFullScreen);
-            if (screen_count == 1)
-            {
-                // Set the chosen screen
-                _container_widget->setGeometry(geom);
-            }
-            _container_widget->setCursor(Qt::BlankCursor);
-            _container_widget->show();
+        if (geom.isNull()) {
+            // Use default screen
+            geom = QApplication::desktop()->screenGeometry(-1);
         }
-        else
-        {
-            // In the dual and multi screen cases we need to bypass the window manager
-            // on X11 because Qt does not support _NET_WM_FULLSCREEN_MONITORS, and thus
-            // the window manager would always restrict the fullscreen window to one screen.
-            // Note: it may be better to set _NET_WM_FULLSCREEN_MONITORS ourselves, but that
-            // would also require the window manager to support this extension...
-            _container_widget->setWindowFlags(_container_widget->windowFlags()
-                    | Qt::X11BypassWindowManagerHint
-                    | Qt::FramelessWindowHint
-                    | Qt::WindowStaysOnTopHint);
-            _container_widget->setWindowState(_widget->windowState() | Qt::WindowFullScreen);
-            _container_widget->setGeometry(geom);
-            _container_widget->setCursor(Qt::BlankCursor);
-            _container_widget->show();
-            _container_widget->raise();
-            _container_widget->activateWindow();
+        Qt::WindowFlags new_window_flags =
+            _container_widget->windowFlags()
+            | Qt::FramelessWindowHint
+            | Qt::WindowStaysOnTopHint;
+        // In the dual and multi screen cases we need to bypass the window manager
+        // on X11 because Qt does not support _NET_WM_FULLSCREEN_MONITORS, and thus
+        // the window manager would always restrict the fullscreen window to one screen.
+        // Note: it may be better to set _NET_WM_FULLSCREEN_MONITORS ourselves, but that
+        // would also require the window manager to support this extension...
+        if (screen_count > 1)
+            new_window_flags |= Qt::X11BypassWindowManagerHint;
+        _container_widget->setWindowFlags(new_window_flags);
+        _container_widget->setWindowState(_container_widget->windowState() | Qt::WindowFullScreen);
+        _container_widget->setGeometry(geom);
+        _container_widget->setCursor(Qt::BlankCursor);
+        _container_widget->show();
+        _container_widget->raise();
+        _container_widget->activateWindow();
 #ifdef Q_WS_X11
-            /* According to the Qt documentation, it should be sufficient to call activateWindow()
-             * to make a X11 window active when using Qt::X11BypassWindowManagerHint, but this
-             * does not work for me (Ubuntu 11.04 Gnome-2D desktop). This is a workaround. */
-            /* Note that using X11 functions directly means that we have to link with libX11
-             * explicitly; see configure.ac. */
+        /* According to the Qt documentation, it should be sufficient to call activateWindow()
+         * to make a X11 window active when using Qt::X11BypassWindowManagerHint, but this
+         * does not work for me (Ubuntu 11.04 Gnome-2D desktop). This is a workaround. */
+        /* Note that using X11 functions directly means that we have to link with libX11
+         * explicitly; see configure.ac. */
+        if (new_window_flags & Qt::X11BypassWindowManagerHint) {
             QApplication::syncX();      // just for safety; not sure if it is necessary
             XSetInputFocus(QX11Info::display(), _container_widget->winId(), RevertToParent, CurrentTime);
             XFlush(QX11Info::display());
-#endif
         }
+#endif
         _container_widget->grab_focus();
         // Suspend the screensaver after going fullscreen, so that our window ID
         // represents the fullscreen window. We need to have the same ID for resume.
@@ -958,24 +905,24 @@ void video_output_qt::enter_fullscreen()
 
 void video_output_qt::exit_fullscreen()
 {
-    if (_fullscreen)
-    {
+    if (_fullscreen) {
         // Resume the screensaver before disabling fullscreen, so that our window ID
         // still represents the fullscreen window and was the same when suspending the screensaver.
         if (_screensaver_inhibited) {
             resume_screensaver();
             _screensaver_inhibited = false;
         }
+        // Re-embed the container widget into the main window if necessary
         if (_container_is_external)
-        {
             _container_widget->setWindowFlags(Qt::Widget);
-        }
         _container_widget->setWindowFlags(_container_widget->windowFlags()
                 & ~Qt::X11BypassWindowManagerHint
                 & ~Qt::FramelessWindowHint
                 & ~Qt::WindowStaysOnTopHint);
-        _container_widget->setWindowState(_widget->windowState() & ~Qt::WindowFullScreen);
+        _container_widget->setWindowState(_container_widget->windowState() & ~Qt::WindowFullScreen);
         _container_widget->setCursor(Qt::ArrowCursor);
+        if (!_container_is_external)
+            _container_widget->setGeometry(_geom);
         _container_widget->show();
         _container_widget->raise();
         _container_widget->grab_focus();
