@@ -50,16 +50,14 @@ private:
     video_output_qt* _vo_qt;
     video_output_qt_widget* _vo_qt_widget;
     bool _render;
-    bool _activate_next_frame;
     bool _resize;
     int _w, _h;
-    bool _prepare_next_frame;
-    bool _have_prepared_frame;
+    mutex _action_mutex;
+    condition _action_cond;
+    bool _action_activate;
+    bool _action_prepare;
     video_frame _next_frame;
-    void* _next_data[2][3];
-    size_t _next_line_size[2][3];
     subtitle_box _next_subtitle;
-    mutex _prepare_next_mutex;
     bool _failure;
     exc _e;
 
@@ -75,15 +73,14 @@ private:
 
 public:
     gl_thread(video_output_qt* vo_qt, video_output_qt_widget* vo_qt_widget);
-    ~gl_thread();
 
 #ifdef Q_WS_X11
     GLXEWContext* glxewGetContext() const;
 #endif
 
     void set_render(bool r);
-    void activate_next_frame();
     void resize(int w, int h);
+    void activate_next_frame();
     void prepare_next_frame(const video_frame &frame, const subtitle_box &subtitle);
 
     int64_t time_to_next_frame_presentation();
@@ -129,7 +126,15 @@ public:
     }
 
 protected:
-    virtual void paintEvent(QPaintEvent*) {};
+#ifdef Q_WS_X11
+# if QT_VERSION < 0x040800
+    /* Work around a bug in Qt < 4.8: On a hide event, the GLWidget calls
+     * makeCurrent() and glFinish(), but this interferes with our GL thread.
+     * This has been fixed in Qt 4.8. */
+    virtual bool event(QEvent* e) { return QWidget::event(e); }
+# endif
+#endif
+    virtual void paintEvent(QPaintEvent*) {}
     virtual void resizeEvent(QResizeEvent* event);
     virtual void keyPressEvent(QKeyEvent *event);
     virtual void mouseReleaseEvent(QMouseEvent *event);
@@ -174,6 +179,7 @@ private:
     video_output_qt_widget *_widget;
     QGLFormat _format;
     bool _fullscreen;
+    QRect _geom;
     bool _screensaver_inhibited;
     bool _recreate_context;
     bool _recreate_context_stereo;
@@ -189,7 +195,6 @@ protected:
     GLXEWContext* glxewGetContext() const;
 #endif
     virtual GLEWContext* glewGetContext() const;
-    virtual void make_context_current();
     virtual bool context_is_stereo() const;
     virtual void recreate_context(bool stereo);
     virtual void trigger_resize(int w, int h);
